
#define LCDPORT portb 
#define LCDTRIS trisb
#define LCD_RS porta.7
#define LCD_TRIS_RS trisa.7
//#define LCD_RW porta.4
//#define LCD_TRIS_RW trisa.4
#define LCD_E portb.1
#define LCD_TRIS_E trisb.1

void PutChar(unsigned char);
void InitLCD(void);
void PutMessage(rom char*);
void PutMessageAt(unsigned char,unsigned char,rom char*);
void PutHex(unsigned int,char);

#define Width 16
#define Height 2

void WriteCommand(unsigned char);
void WriteChar(unsigned char);
void WriteNibble(unsigned char);
void WaitLCDBusy(void);
//void WriteLCD(unsigned char);
void PutNibble(unsigned char);
void ScrollDisplay();

unsigned char Xpos,Ypos;

//Write a string to the LCD
void PutMessage(rom char *Message){
char i=0;
    while(Message[i]!=0)
        PutChar(Message[i++]);
} 

void PutBuffer(char *Message){
char i=0;
    while(Message[i]!=0)
        PutChar(Message[i++]);
} 

void SetPos(unsigned char X,unsigned char Y){
    Xpos=X;
    Ypos=Y;
    if(Y&1)
        X+=0x40;
    if(Y&2)
        X+=Width;
    WriteCommand(0x80+X);    
}

//Set position and write string.
void PutMessageAt(unsigned char X,unsigned char Y,rom char *Message){
char i=0;
    SetPos(X,Y);
    while(Message[i]!=0)
        PutChar(Message[i++]);
}

void PutHex(unsigned int Num,char Digits){
char i;
    Num<<=(4-Digits)*4;
    for(i=0;i<Digits;i++){
        PutNibble((Num>>12)&0x0f);
        Num<<=4;
    }
}

void PutDecimal(unsigned char num){
    PutChar((num/10)+0x30);
    PutChar((num%10)+0x30);
}

void PutChar(unsigned char chr){
    #ifdef LCD_RW
    if(Ypos>(Height-1)){
        ScrollDisplay();
        Ypos=Height-1;
    }
    #endif
    WriteChar(chr);
    if(++Xpos==Width){
        Xpos=0;
        Ypos++;
        SetPos(Xpos,Ypos);
    }
}

void PutNibble(unsigned char num){
    if (num>9){
        WriteChar(num+'A'-10);
    }else{
        WriteChar(num+'0');
    }
}

//Initialises the LCD
void InitLCD(void){
    LCDTRIS &= 0xc3;                //ensure data bits are output
    LCD_E=0;                        //clear enable
    LCD_RS = 0;                     //going to write command
    LCD_TRIS_E=0;                   //Set enable to output
    LCD_TRIS_RS=0;                  //set RS to output
    #ifdef LCD_RW
        LCD_TRIS_RW=0;
        LCD_RW=0;
    #endif
    delay_ms(30);                   //delay for LCD to initialise.
    WriteNibble(0x30);              //Required for initialisation
    delay_ms(5);                    //required delay
    WriteNibble(0x30);              //Required for initialisation
    delay_ms(1);                    //required delay
    WriteCommand(0x20);             //set to 4 bit interface
    WriteCommand(0x2c);             //set to 4 bit interface, 2 line and 5*10 font
    WriteCommand(0x01);             //clear display
    WriteCommand(0x06);             //move cursor right after write
    WriteCommand(0x0C);             //turn on display
    SetPos(0,0);
}

void CursorOn(){
    WriteCommand(0x0C+2);           //turn on cursor
}

void CursorOff(){
    WriteCommand(0x0C);             //turn off cursor
}

#ifdef LCD_RW
//wait for the LCD to finish last command.
void WaitLCDBusy(void){
    LCD_RS=0;
    LCD_RW=1;
    LCDTRIS|=0x3c;
    LCD_E=1;
    while(LCDPORT.5==1);
    LCD_E=0;
    LCDTRIS&=0xc3;
    LCD_RW=0;
}
#endif

//Send a command to the LCD
void WriteCommand(unsigned char command){
    #ifdef LCD_RW
    WaitLCDBusy();                  //wait until not busy
    #endif
    LCD_RS = 0;                     //setup to send command
    WriteNibble(command>>4);        //write the high nibble
    WriteNibble(command);           //then the low nibble
    #ifndef LCD_RW
    if(command&0xfc)
        delay_us(50);
    else
        delay_ms(2);
    #endif
}

//Send a character to the LCD
void WriteChar(unsigned char chr){
    #ifdef LCD_RW
    WaitLCDBusy();                  //wait until not busy
    #endif
    LCD_RS=1;                       //Setup to send character
    WriteNibble(chr>>4);            //write the high nibble
    WriteNibble(chr);               //then the low nibble
    #ifndef LCD_RW
    delay_us(50);
    #endif
}

//Send any 4 bits to the LCD
void WriteNibble(unsigned char command){
    LCDPORT &= 0xc3;                //clear the data bits
    LCDPORT|=((command & 0x0f)<<2); //or in the new data
    LCD_E = 1;                      //enable the LCD interface
    LCD_E = 0;                      //disable it
}

#ifdef LCD_RW
unsigned char ReadNibble(){
char chr;
    LCD_RS=1;
    LCD_RW=1;
    LCDTRIS|=0x3c;
    LCD_E=1;
    chr=(LCDPORT&0x3c)>>2;
    LCD_E=0;
    LCDTRIS&=0xc3;
    LCD_RW=0;
    return chr;
}

char ReadChar(){
char chr;
    chr=ReadNibble()<<4;
    chr|=ReadNibble();
    return chr;
}

void ScrollDisplay(){
char i,j,chr;
    for(i=0;i<20;i++){
        for(j=0;j<3;j++){
            SetPos(i,j+1);
            chr=ReadChar();
            SetPos(i,j);
            WriteChar(chr);
        }
    }
    SetPos(0,3);
    for(i=0;i<20;i++)
        WriteChar(' ');
    SetPos(0,3);
}
#endif

