
#define A_None      1
#define A_SubMenu   2

#define T_text      1
#define T_yesno     2
#define T_truefalse 3
#define T_display   4
#define T_input     5
#define T_multi     6
#define T_display32 7
#define T_text_disp 8
#define T_sp_va     9
#define T_onoff     10

#define NumDataBytes 3

void ShowMenu(unsigned char,unsigned char,unsigned char);
unsigned char ReadFlash(unsigned int);
unsigned char CountMenuItems(unsigned char);
unsigned char ReadEEPROM(unsigned char);
void WriteEEPROM(unsigned char, unsigned char);

unsigned char menuStack[16],menuPointer,currentMenu,selectedMenu;
unsigned char menuType,menuData[3];
unsigned int pointer;
unsigned char menuVars[4];

void FindFirst(unsigned char menuNum){
    pointer=0x1000;                         //point to start of data
    while(ReadFlash(pointer)!=menuNum){     //while not menu number
        pointer+=NumDataBytes+2;            //skip data bytes
        while(ReadFlash(pointer++));        //skip string
    }                                       //note, no sanity checks
}

void EnterMenu(unsigned char menuNum){
unsigned char i;
    FindFirst(menuNum);                     //point to first entry in this menu
    menuType=ReadFlash(++pointer);          //get the type
    for(i=0;i<NumDataBytes;i++)             //copy the 3 data bytes
        menuData[i]=ReadFlash(++pointer);
    for(i=0;i<4;i++){                       //and copy 4 EEPROM locations
        menuVars[i]=ReadEEPROM(menuData[0]+i);
    }
}

void ExitMenu(){
unsigned char i;
    switch(menuType){
        case T_onoff:
        case T_yesno:
        case T_truefalse:
        case T_input:
        case T_multi:
            for(i=0;i<4;i++){                       //write back any locations that have changed
                if((menuVars[i])!=ReadEEPROM(menuData[0]+i))
                    WriteEEPROM(menuData[0]+i,menuVars[i]);
            }
    }
}

unsigned char CountMenuItems(unsigned char menuNum){
unsigned char i;
    FindFirst(menuNum);                     //counts the number of entries in
    i=0;                                    //the current menu
    while(ReadFlash(pointer)==menuNum){
        pointer+=NumDataBytes+2;
        while(ReadFlash(pointer++));
        i++;
    }
    return(i);
}

/************************************************
ShowMenu will display a menu on the LCD
    menuNumber is the number of the menu to show
    first is which menu item to display on the top line
    highlight is which screen line is selected
ShowMenu(5,2,0) will show menu 5, items 2 and 3, with the item on the top line selected
************************************************/
void ShowMenu(unsigned char menuNum,unsigned char first,unsigned char highlight){
unsigned char data,temp,i,j;
    FindFirst(menuNum);                     //point at first entry
    while(--first){                         //skip entries that are off screen
        pointer+=NumDataBytes+2;            //skip the data
        while(ReadFlash(pointer++));        //skip the text
    }
    for(i=0;i<2;i++){                       //display next 2 entries
        SetPos(0,i);
        if(highlight==i)                    //if it is the selected entry
            PutChar('>');                   //put a >
        else
            PutChar(' ');                   //else put a space
        for(j=1;j<16;j++)
            PutChar(' ');                   //blank rest of line
        if(ReadFlash(pointer++)!=menuNum){  //if no second entry
            break;                          //get out of for loop
        }
        menuType=ReadFlash(pointer++);      //get menu type
        temp=ReadFlash(pointer);
        if(highlight==i){                   //if selected menu, copy data
            for(j=0;j<NumDataBytes;j++)
                menuData[j]=ReadFlash(pointer++);
        }else
            pointer+=NumDataBytes;          //otherwise skip data
        SetPos(0,i);                        //and prepare to use whole line
        if((menuType==T_text)               //if type text 
                ||(menuType==T_multi))      //or type multi
            SetPos(1,i);                    //then dont overwrite > char
        while(data=ReadFlash(pointer++)){   //display text
            if(data==0xff)
                PutHex(ReadEEPROM(temp),2);
            else
                PutChar(data);
        }
    }
}


unsigned char ReadFlash(unsigned int offset){
    eeadr=offset&255;
    eeadrh=(offset>>8);
    eecon1.EEPGD=1;
    eecon1.RD=1;
    asm{
        nop
        nop
    }
    return(eedata);    
}

unsigned char ReadEEPROM(unsigned char add){
    eeadr=add;
    eecon1.EEPGD=0;
    eecon1.RD=1;
    return(eedata);    
}

void WriteEEPROM(unsigned char add, unsigned char dat){
    while(eecon1.WR);
    eeadr=add;
    eedata=dat;
    eecon1.EEPGD=0;
    eecon1.WREN=1;
    eecon2=0x55;
    eecon2=0xaa;
    eecon1.WR=1;
    eecon1.WREN=0;
}

/****************************************************
Each entry is,
    Menu Number
    Menu Type
    Menu Data - 3 bytes of data
    Menu String - what's displayed
*****************************************************/

/****************************************************
Types of menu items.

T_text - just a text item
    Data1 = action to take if selected - eg A_SubMenu
    Data2 = sub menu number
    Data3 N/A

T_yesno - displays text and yes/no
    Data1 = address in EEPROM of bit
    Data2 = bit number
    data3 N/A

T_truefalse - displays text and true/false
    Data1 = address in EEPROM of bit
    Data2 = bit number
    data3 N/A

T_display - displays a 4 digit variable
    Data1 = address in EEPROM of data
    Data2 = digits before decimal point
    Data3 = digits after decimal point

T_input - displays a 4 digit variable and allows it to be edited
    Data1 = address in EEPROM of data
    Data2 = digits before decimal point
    Data3 = digits after decimal point

T_multi - displays multi choice menu
    Data1 = address in EEPROM of data
    Data2 N/A
    Data3 N/A


*****************************************************/





#pragma DATA 0x2100 ,0b00000000                             //8 bit vars
                    ,0x37,0x43,0x41,0x45,0x34,0x50,0x27     //history
                    ,0x00,0x12,0x34,0x56                    //Total today
                    ,0x00,0x00,0x43,0x21                    //Total today
                    ,0x03                                   //baud
                    ,0x01                                   //voltage

#define L_flags     0x00
#define L_today     0x01
#define L_today1    0x02
#define L_today2    0x03
#define L_today3    0x04
#define L_today4    0x05
#define L_today5    0x06
#define L_today6    0x07
#define L_gen       0x08    //4
#define L_dump      0x0c    //4
#define L_baud      0x10
#define L_volt      0x11


#pragma DATA 0x1000 ,1,T_text,A_None,0,0,"Main Menu"
                    ,1,T_text,A_SubMenu,50,0,"Display"
                    ,1,T_text,A_SubMenu,100,0,"Control"
                    ,1,T_text,A_SubMenu,150,0,"Options"

                    ,50,T_text,A_SubMenu,60,0,"Volts Amps in"
                    ,50,T_text,A_SubMenu,65,0,"Volts Amps out"
                    ,50,T_text,A_SubMenu,70,0,"Generated Today"
                    ,50,T_text,A_SubMenu,75,0,"Dumped Today"
                    ,50,T_text,A_SubMenu,80,0,"weekly Stats"

                    ,60,T_sp_va,1,0,0,"Volts Amps in"
                    ,65,T_sp_va,2,0,0,"Volts Amps out"

                    ,70,T_display32,L_gen,6,2,"Generated Today"
                    ,75,T_display32,L_dump,6,2,"Dumped Today"
                    ,80,T_text,L_today,4,0,"Today   \xffkWh"
                    ,80,T_text,L_today1,4,0,"Today-1 \xffkWh"
                    ,80,T_text,L_today2,4,0,"Today-2 \xffkWh"
 
                    ,100,T_text,A_None,4,0,"Control"
                    ,100,T_text,A_SubMenu,101,0,"Charging On/Off"
                    ,100,T_text,A_SubMenu,102,0,"Allow Dump"

                    ,150,T_text,0,4,0,"Options"
                    ,150,T_text,A_SubMenu,151,0,"Set Baud Rate"
                    ,150,T_text,A_SubMenu,152,0,"Set Voltage"

                    ,101,T_onoff,L_flags,0,0,"Charging"
                    ,102,T_truefalse,L_flags,1,0,"Allow Dump"

                    ,151,T_multi,L_baud,0,0,"2400"
                    ,151,T_multi,L_baud,0,0,"4800"
                    ,151,T_multi,L_baud,0,0,"9600"
                    ,151,T_multi,L_baud,0,0,"19200"

                    ,152,T_multi,L_baud,0,0,"Voltage 12V"
                    ,152,T_multi,L_baud,0,0,"Voltage 24V"
                    ,152,T_multi,L_baud,0,0,"Voltage 36V"
                    ,0xff