
#include <system.h>
#include "LCD.c"
#include "Keys.c"
#include "Menu.c"

#pragma DATA _CONFIG, _DEBUG_OFF & _MCLRE_ON
                    & _CP_OFF & _PWRTE_OFF
                    & _WDT_OFF & _INTOSCIO
                    & _LVP_OFF

#pragma CLOCK_FREQ 8000000

void Navigate(unsigned char);

char PK2_Reserved @0x70;
unsigned char screenMenu;
bit redraw,newMenu;


void main(void){
unsigned char key,i,j,digit,buffer[17],*p; 
bit flag;
    while(!osccon.HTS);
    osccon=0x71;            //8 meg
    ansel=0;
    anselh=0;
    option_reg=0b00000000;  //wpu on
    InitLCD();
    menuPointer=0;
    currentMenu=1;
    selectedMenu=1;
    screenMenu=1;
    menuType=0;
    redraw=1;
    newMenu=1;
    while(1){
        if(newMenu){
            ExitMenu();
            CursorOff();
            EnterMenu(currentMenu);
            digit=0;
            newMenu=0;
            switch(menuType){
                case T_input:
                case T_display:
                case T_display32:
                    i=0;                        //expand BCD number into buffer
                    for(j=0;j<4;j++){
                       buffer[i++]=(menuVars[j]>>4)+'0';
                       buffer[i++]=(menuVars[j]&0x0f)+'0';
                    }
                    if(menuType==T_display32)
                        buffer[8]=0;            //is an 8 digit number
                    else
                        buffer[4]=0;            //is a 4 digit number
                    break;
                case T_multi:
                    selectedMenu=menuVars[0];
                    break;
            }
        }                
        if(redraw){
            redraw=0;
            while(selectedMenu>(screenMenu+1))  //ensure menu is on screen
                screenMenu++;
            while(selectedMenu<screenMenu)
                screenMenu--;
            ShowMenu(currentMenu,screenMenu,selectedMenu-screenMenu);
            //now do any extras for different menu types
            flag=menuVars[0]>>menuData[1];      //copy any bit variable into flag
            switch(menuType){
                case T_yesno:
                    if(flag)
                      PutMessageAt(7,1,"Yes");
                    else
                        PutMessageAt(7,1,"No ");
                    break;
                case T_truefalse:
                    if(flag)
                        PutMessageAt(6,1,"True ");
                    else
                        PutMessageAt(6,1,"False");
                    break;
                case T_onoff:
                    if(flag)
                        PutMessageAt(7,1,"On ");
                    else
                        PutMessageAt(7,1,"Off");
                    break;
                case T_input:
                    CursorOn();
                case T_display:
                case T_display32:
                    SetPos(7-(menuData[1]>>1),1);
                    for(j=0;j<(menuData[1]+menuData[2]);j++){
                        if(j==menuData[1])
                            PutChar('.');
                        PutChar(buffer[j]);
                    }
                    if(digit>=menuData[1])
                        SetPos(8-(menuData[1]>>1)+digit,1);
                    else
                        SetPos(7-(menuData[1]>>1)+digit,1);
                    break;
                case T_sp_va:
                    if(menuData[0]==1)
                        //read actual values here and display them
                        PutMessageAt(2,1,"8.1V 2.3A");
                    else
                        PutMessageAt(2,1,"12.2V 1.5A");
                    break;
            }
        }
        delay_ms(10);
        key=ReadKeyPad();
        switch(menuType){
            case T_text:
            case T_display:         //all of these menu types
            case T_display32:       //just require navigation
            case T_multi:
            case T_sp_va:
                Navigate(key);
                break;
            case T_onoff:
            case T_yesno:           //these require a bit to be flipped
            case T_truefalse:
                switch(key){
                    case key_none:
                        break;
                    case key_back:
                    case key_enter:
                        selectedMenu=menuStack[--menuPointer];
                        currentMenu=menuStack[--menuPointer];
                        redraw=1;
                        newMenu=1;
                        break;
                    default:        //up or down pressed
                        menuVars[0]^=(1<<menuData[1]);
                        redraw=1;
                }
                break;
            case T_input:
                switch(key){
                    case key_up:
                        if(buffer[digit]<9)     //increment digit
                            buffer[digit]++;
                        redraw=1;
                        break;
                    case key_dw:                //decrement digit
                        if(buffer[digit]>0)
                            buffer[digit]--;
                        redraw=1;
                        break;
                    case key_enter:             //move to next digit
                        if(++digit>=(menuData[1]+menuData[2]))
                            digit=0;
                        redraw=1;
                        break;
                    case key_back:              //done so
                        i=0;                    //convert buffer back to packed BCD
                        for(j=0;j<2;j++){
                            menuVars[j]=(buffer[i++]-'0')<<4;
                            menuVars[j]|=buffer[i++]-'0';
                        }
                        selectedMenu=menuStack[--menuPointer];  //pop menu
                        currentMenu=menuStack[--menuPointer];
                        redraw=1;
                        newMenu=1;
                        break;
                }
                break;
        }
    }
}


void Navigate(unsigned char key){
    switch(key){
        case key_back:
            //was back button so pop menu off stack
            if(menuType==T_multi)
                menuVars[0]=selectedMenu;
            if(menuPointer>0){
                selectedMenu=menuStack[--menuPointer];
                currentMenu=menuStack[--menuPointer];
                redraw=1;
                newMenu=1;
            }
            break;
        case key_up:
            //was up button
            if(selectedMenu>1){
                selectedMenu--;
                redraw=1;
            }
            break;
        case key_dw:
            //was down button
            if(selectedMenu<CountMenuItems(currentMenu)){
                selectedMenu++;
                redraw=1;
            }
            break;
        case key_enter:
            //was enter button so navigate to sub menu if one exists
            if(menuData[0]==A_SubMenu){
                menuStack[menuPointer++]=currentMenu;   //push current menu
                menuStack[menuPointer++]=selectedMenu;
                currentMenu=menuData[1];                //and get new one
                selectedMenu=1;
                screenMenu=1;
                redraw=1;
                newMenu=1;
            }
            break;
    }
}

